@@ -739,10 +739,10 @@ static void gfsd_update_preview (GnomeFontSelection * fs, GnomeFont * font, Gnom
 
 static GtkDialogClass *gfsd_parent_class = NULL;
 
-guint
+GtkType
 gnome_font_dialog_get_type (void)
 {
-	static guint font_dialog_type = 0;
+	static GtkType font_dialog_type = 0;
 	if (!font_dialog_type) {
 		GtkTypeInfo fontsel_diag_info = {
 			"GnomeFontDialog",
