@@ -47,8 +47,6 @@ struct _GnomePrintCopiesSelection {
 	GtkWidget *copies;
 	GtkWidget *collate;
 	GtkWidget *collate_image;
-
-	GtkAccelGroup *accel_group;
 };
 
 struct _GnomePrintCopiesSelectionClass {
@@ -64,7 +62,7 @@ static void gnome_print_copies_selection_destroy (GtkObject *object);
 /* again, these images may be here temporarily */
 
 /* XPM */
-static char * collate_xpm[] = {
+static const char *collate_xpm[] = {
 "65 35 6 1",
 " 	c None",
 ".	c #000000",
@@ -109,7 +107,7 @@ static char * collate_xpm[] = {
 "....................              ....................           "};
 
 /* XPM */
-static char * nocollate_xpm[] = {
+static const char *nocollate_xpm[] = {
 "65 35 6 1",
 " 	c None",
 ".	c #000000",
@@ -203,16 +201,14 @@ collate_toggled (GtkWidget *widget, GnomePrintCopiesSelection *gpc)
 {
 	gint copies;
 	gboolean collate;
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
+	GdkPixbuf *pb;
 
 	copies = gtk_spin_button_get_value_as_int ((GtkSpinButton *) gpc->copies);
 	collate = ((GtkToggleButton *) gpc->collate)->active;
 
-	pixmap = gdk_pixmap_create_from_xpm_d (NULL, &mask, NULL, collate?collate_xpm:nocollate_xpm);
-	gtk_image_set_from_pixmap (GTK_IMAGE (gpc->collate_image), pixmap, mask);
-	gdk_pixmap_unref (pixmap);
-	gdk_bitmap_unref (mask);
+	pb = gdk_pixbuf_new_from_xpm_data (collate ? collate_xpm : nocollate_xpm);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (gpc->collate_image), pb);
+	gdk_pixbuf_unref (pb);
 
 	if (gpc->changing) return;
 
@@ -241,10 +237,12 @@ gnome_print_copies_selection_init (GnomePrintCopiesSelection *gpc)
 	GtkAdjustment *adj;
 	guint label_key;
 	guint collate_key;
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
+	GdkPixbuf *pb;
 
+#if 0
 	gpc->accel_group = gtk_accel_group_new ();
+#endif
+
 	frame = gtk_frame_new (_("Copies"));
 	gtk_container_add (GTK_CONTAINER (gpc), frame);
 	gtk_widget_show (frame);
@@ -264,28 +262,31 @@ gnome_print_copies_selection_init (GnomePrintCopiesSelection *gpc)
 	gpc->copies = gtk_spin_button_new(adj, 1.0, 0);
 	gtk_widget_show (gpc->copies);
 	gtk_table_attach_defaults ((GtkTable *)table, gpc->copies, 1, 2, 0, 1);
+#if 0
 	if (label_key != GDK_VoidSymbol) {
 		gtk_widget_add_accelerator
 			(GTK_WIDGET (&GTK_SPIN_BUTTON (gpc->copies)->entry),
 			 "grab_focus", gpc->accel_group, label_key,
 			 GDK_MOD1_MASK, 0);
 	}
+#endif
 
-	pixmap = gdk_pixmap_create_from_xpm_d (NULL, &mask, NULL, nocollate_xpm);
-	gpc->collate_image = gtk_image_new_from_pixmap (pixmap, mask);
-	gdk_pixmap_unref (pixmap);
-	gdk_bitmap_unref (mask);
+	pb = gdk_pixbuf_new_from_xpm_data (nocollate_xpm);
+	gpc->collate_image = gtk_image_new_from_pixbuf (pb);
+	gdk_pixbuf_unref (pb);
 	gtk_widget_show (gpc->collate_image);
 	gtk_table_attach_defaults ((GtkTable *)table, gpc->collate_image, 0, 1, 1, 2);
 
 	gpc->collate = gtk_check_button_new_with_label("");
 	collate_label = GTK_LABEL (GTK_BIN (gpc->collate)->child);
 	collate_key = gtk_label_parse_uline (collate_label, _("_Collate"));
+#if 0
 	if (collate_key != GDK_VoidSymbol) {
 		gtk_widget_add_accelerator (gpc->collate, "clicked",
 					   gpc->accel_group, collate_key,
 					   GDK_MOD1_MASK, 0);
 	}
+#endif
 
 	gtk_widget_show(gpc->collate);
 	gtk_table_attach_defaults((GtkTable *)table, gpc->collate, 1, 2, 1, 2);
@@ -341,8 +342,10 @@ gnome_print_copies_selection_destroy (GtkObject *object)
 	
 	gpc = GNOME_PRINT_COPIES_SELECTION (object);
 
+#if 0
 	if (gpc->accel_group)
 		gtk_accel_group_unref (gpc->accel_group);
+#endif
   
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
