@@ -30,6 +30,8 @@
 #include <time.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkdialog.h>
+#include <gtk/gtknotebook.h>
+#include <gtk/gtkvbox.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtkradiobutton.h>
@@ -42,20 +44,21 @@
 #include <libgnomeprint/gnome-print-config.h>
 
 #include "gnome-printer-dialog.h"
+#include "gnome-print-paper-selector.h"
 #include "gnome-print-copies.h"
 #include "gnome-print-dialog.h"
 
+#define PAD 4
+
 struct _GnomePrintDialog {
 	GtkDialog dialog;
 
-	void *priv;
+	GnomePrintConfig *config;
 
+	GtkWidget *notebook;
+
+	GtkWidget *job;
 	GtkWidget *printer;
-	GtkWidget *copies;
-	GnomePrintRangeType range_type;
-	GtkWidget *range;
-	GtkWidget *range_container;
-	GtkAccelGroup *range_accel_group;
 };
 
 struct _GnomePrintDialogClass {
@@ -65,19 +68,10 @@ struct _GnomePrintDialogClass {
 static void gnome_print_dialog_class_init (GnomePrintDialogClass *class);
 static void gnome_print_dialog_init (GnomePrintDialog *gspaper);
 
+static GtkWidget *gpd_create_job_page (void);
+
 static GtkDialog *parent_class;
 
-typedef struct _GnomePrintDialoguePrivate {
-	int flags;
-
-	GtkWidget *current;
-	GtkWidget *all;
-	GtkWidget *range;
-	GtkWidget *selection;
-	GtkWidget *range_start;
-	GtkWidget *range_end;
-} Private;
-
 GtkType
 gnome_print_dialog_get_type (void)
 {
@@ -105,12 +99,8 @@ gnome_print_dialog_destroy (GtkObject *object)
 	
 	gpd = GNOME_PRINT_DIALOG (object);
 
-	if (gpd->range_accel_group) {
-		gtk_accel_group_unref (gpd->range_accel_group);
-	}
-
-	if (gpd->priv) {
-		g_free (gpd->priv);
+	if (gpd->config) {
+		gpd->config = gnome_print_config_unref (gpd->config);
 	}
 
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
@@ -131,63 +121,120 @@ gnome_print_dialog_class_init (GnomePrintDialogClass *class)
 static void
 gnome_print_dialog_init (GnomePrintDialog *gpd)
 {
-	gpd->printer = gnome_printer_selection_new_default ();
+	gpd->config = gnome_print_config_default ();
 
-	if (gpd->printer) {
-		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (gpd)->vbox), GTK_WIDGET (gpd->printer), FALSE, FALSE, 3);
+	if (gpd->config) {
+		GtkWidget *hb, *vb, *p, *l;
+		gpd->notebook = gtk_notebook_new ();
+		gtk_widget_show (gpd->notebook);
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (gpd)->vbox), gpd->notebook, TRUE, TRUE, 0);
+		/* Job page */
+		gpd->job = gpd_create_job_page ();
+		gtk_widget_show (gpd->job);
+		l = gtk_label_new (_("Job"));
+		gtk_widget_show (l);
+		gtk_notebook_append_page (GTK_NOTEBOOK (gpd->notebook), gpd->job, l);
+		/* Printers page */
+		hb = gtk_hbox_new (FALSE, 0);
+		gtk_widget_show (hb);
+		vb = gtk_vbox_new (FALSE, 0);
+		gtk_widget_show (vb);
+		gtk_box_pack_start (GTK_BOX (hb), vb, FALSE, FALSE, 0);
+		gpd->printer = gnome_printer_selection_new (gpd->config);
 		gtk_widget_show (gpd->printer);
-	} else
-		g_warning ("There aren't any printers installed\n");
-
-	
-	gpd->range_type = GNOME_PRINT_RANGETYPE_NONE;
-	gpd->priv = g_new0 (Private, 1);
-	gpd->range_accel_group = NULL;
+		gtk_box_pack_start (GTK_BOX (vb), gpd->printer, FALSE, FALSE, 0);
+		l = gtk_label_new (_("Printer"));
+		gtk_widget_show (l);
+		gtk_notebook_append_page (GTK_NOTEBOOK (gpd->notebook), hb, l);
+		/* Paper page */
+		p = gnome_paper_selector_new (gpd->config);
+		gtk_widget_show (p);
+		l = gtk_label_new (_("Paper"));
+		gtk_widget_show (l);
+		gtk_notebook_append_page (GTK_NOTEBOOK (gpd->notebook), p, l);
+	} else {
+		GtkWidget *label;
+		label = gtk_label_new (_("Error in loading printer configuration"));
+		gtk_widget_show (label);
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (gpd)->vbox), label, TRUE, TRUE, 0);
+	}
 }
 
-#define PAD 4
-static void
-init_body (GnomePrintDialog *gpd, int flags)
+static GtkWidget *
+gpd_create_job_page (void)
 {
-	GtkWidget *hbox;
-	GnomePrinterSelection *gpw;
+	GtkWidget *hb, *vb, *f, *c;
 
-	gpw = GNOME_PRINTER_SELECTION (gpd->printer);
+	hb = gtk_hbox_new (FALSE, 0);
 
-	if (gpd->copies || gpd->range_container) return;
+	vb = gtk_vbox_new (FALSE, PAD);
+	gtk_widget_show (vb);
+	gtk_box_pack_start (GTK_BOX (hb), vb, FALSE, FALSE, PAD);
 
-	if (flags & (GNOME_PRINT_DIALOG_RANGE | GNOME_PRINT_DIALOG_COPIES)) {
-		hbox = gtk_hbox_new (FALSE, 3);
-		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (gpd)->vbox), GTK_WIDGET (hbox), FALSE, FALSE, 3);
+	f = gtk_frame_new (_("Print Range"));
+	gtk_widget_hide (f);
+	gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, PAD);
+	gtk_object_set_data (GTK_OBJECT (hb), "range", f);
 
-		if (flags & GNOME_PRINT_DIALOG_COPIES) {
-			GnomePrintCopiesSelection *gpc;
-			gpd->copies = gnome_print_copies_selection_new ();
-			gtk_box_pack_end (GTK_BOX (hbox), gpd->copies, FALSE, TRUE, 3);
-			gpc = GNOME_PRINT_COPIES_SELECTION (gpd->copies);
-#if 0
-			gnome_print_copies_bind_editable_enters (gpc, GNOME_DIALOG (gpd));
-			gnome_print_copies_bind_accel_group (gpc, GTK_WINDOW (gpd));
-#endif
-		}
-		
-		if (flags & GNOME_PRINT_DIALOG_RANGE) {
-			gpd->range_container = gtk_frame_new (_("Print Range"));
-			gtk_box_pack_start(GTK_BOX(hbox), gpd->range_container, FALSE, TRUE, 3);
-			gtk_widget_show(gpd->range_container);
-			gpd->range_accel_group = gtk_accel_group_new ();
-			gtk_window_add_accel_group (GTK_WINDOW (gpd), gpd->range_accel_group);
-		}
-		gtk_widget_show_all(hbox);
+	c = gnome_print_copies_selection_new ();
+	gtk_widget_hide (c);
+	gtk_box_pack_start (GTK_BOX (vb), c, FALSE, FALSE, PAD);
+	gtk_object_set_data (GTK_OBJECT (hb), "copies", c);
+
+	return hb;
+}
+
+static GtkWidget *
+gpd_create_range (gint flags, GtkWidget *range, const guchar *clabel, const guchar *rlabel)
+{
+	GtkWidget *t, *rb;
+	GSList *group;
+	gint row;
+
+	t = gtk_table_new (4, 2, FALSE);
+
+	group = NULL;
+	row = 0;
+
+	if (flags & GNOME_PRINT_RANGE_CURRENT) {
+		rb = gtk_radio_button_new_with_label (group, clabel);
+		gtk_object_set_data (GTK_OBJECT (t), "current", rb);
+		gtk_widget_show (rb);
+		gtk_table_attach (GTK_TABLE (t), rb, 0, 1, row, row + 1, GTK_FILL, GTK_FILL, 0, 0);
+		group = gtk_radio_button_group (GTK_RADIO_BUTTON (rb));
+		row += 1;
 	}
 
-#if 0
-	gnome_printer_widget_bind_editable_enters (gpw, GNOME_DIALOG (gpd));
-	gnome_printer_widget_bind_accel_group (gpw, GTK_WINDOW (gpd));
-#endif
-#if 0
-	gtk_widget_grab_focus (gpw->profile_selector);
-#endif
+	if (flags & GNOME_PRINT_RANGE_ALL) {
+		rb = gtk_radio_button_new_with_label(group, _("All"));
+		gtk_object_set_data (GTK_OBJECT (t), "all", rb);
+		gtk_widget_show (rb);
+		gtk_table_attach (GTK_TABLE (t), rb, 0, 1, row, row + 1, GTK_FILL, GTK_FILL, 0, 0);
+		group = gtk_radio_button_group (GTK_RADIO_BUTTON (rb));
+		row += 1;
+	}
+
+	if (flags & GNOME_PRINT_RANGE_RANGE) {
+		rb = gtk_radio_button_new_with_label (group, rlabel);
+		gtk_object_set_data (GTK_OBJECT (t), "range", rb);
+		gtk_widget_show (rb);
+		gtk_table_attach (GTK_TABLE (t), rb, 0, 1, row, row + 1, GTK_FILL, GTK_SHRINK, 0, 0);
+		gtk_table_attach (GTK_TABLE (t), range, 1, 2, row, row + 1, GTK_FILL, 0, 0, 0);
+		group = gtk_radio_button_group (GTK_RADIO_BUTTON (rb));
+		row += 1;
+	}
+
+	if ((flags & GNOME_PRINT_RANGE_SELECTION) || (flags & GNOME_PRINT_RANGE_SELECTION_UNSENSITIVE)) {
+		rb = gtk_radio_button_new_with_label (group, _("Selection"));
+		gtk_object_set_data (GTK_OBJECT (t), "selection", rb);
+		gtk_widget_show (rb);
+		gtk_widget_set_sensitive (rb, !(flags & GNOME_PRINT_RANGE_SELECTION_UNSENSITIVE));
+		gtk_table_attach (GTK_TABLE (t), rb, 0, 1, row, row + 1, GTK_FILL, GTK_FILL, 0, 0);
+		group = gtk_radio_button_group (GTK_RADIO_BUTTON (rb));
+		row += 1;
+	}
+
+	return t;
 }
 
 /**
@@ -211,11 +258,9 @@ gnome_print_dialog_new (const guchar *title, gint flags)
 {
 	GtkWidget *w;
 
-	w = GTK_WIDGET (gtk_type_new (GNOME_TYPE_PRINT_DIALOG));
+	w = gtk_type_new (GNOME_TYPE_PRINT_DIALOG);
 
-	if (GNOME_PRINT_DIALOG (w)->printer == NULL) return NULL;
-
-	gnome_print_dialog_construct (w, title, flags);
+	if (w) gnome_print_dialog_construct (GNOME_PRINT_DIALOG (w), title, flags);
 
 	return w;
 }
@@ -230,9 +275,12 @@ gnome_print_dialog_new (const guchar *title, gint flags)
  **/
 
 void
-gnome_print_dialog_construct (GtkWidget *gpd, const guchar *title, gint flags)
+gnome_print_dialog_construct (GnomePrintDialog *gpd, const guchar *title, gint flags)
 {
-	gtk_window_set_title (GTK_WINDOW (gpd), title);
+	g_return_if_fail (gpd != NULL);
+	g_return_if_fail (GNOME_IS_PRINT_DIALOG (gpd));
+
+	gtk_window_set_title (GTK_WINDOW (gpd), (title) ? title : (const guchar *) _("Gnome Print Dialog"));
 
 	gtk_dialog_add_buttons (GTK_DIALOG (gpd),
 				_("Print"), GNOME_PRINT_DIALOG_RESPONSE_PRINT,
@@ -240,7 +288,18 @@ gnome_print_dialog_construct (GtkWidget *gpd, const guchar *title, gint flags)
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				NULL);
 
-	init_body (GNOME_PRINT_DIALOG (gpd), flags);
+	/* Construct job options, if needed */
+	if (flags & GNOME_PRINT_DIALOG_RANGE) {
+		GtkWidget *r;
+		r = gtk_object_get_data (GTK_OBJECT (gpd->job), "range");
+		if (r) gtk_widget_show (r);
+	}
+
+	if (flags & GNOME_PRINT_DIALOG_COPIES) {
+		GtkWidget *c;
+		c = gtk_object_get_data (GTK_OBJECT (gpd->job), "copies");
+		if (c) gtk_widget_show (c);
+	}
 }
 
 /**
@@ -254,27 +313,21 @@ gnome_print_dialog_construct (GtkWidget *gpd, const guchar *title, gint flags)
 void
 gnome_print_dialog_construct_range_custom (GnomePrintDialog *gpd, GtkWidget *custom)
 {
+	GtkWidget *f, *r;
+
 	g_return_if_fail (gpd != NULL);
 	g_return_if_fail (GNOME_IS_PRINT_DIALOG (gpd));
-	g_return_if_fail (gpd->range_container != NULL);
+	g_return_if_fail (custom != NULL);
+	g_return_if_fail (GTK_IS_WIDGET (custom));
 
-	if (gpd->range) {
-		gtk_container_remove (GTK_CONTAINER (gpd->range_container), gpd->range);
-		gtk_object_unref (GTK_OBJECT(gpd->range));
-		gpd->range = NULL;
-	}
+	f = gtk_object_get_data (GTK_OBJECT (gpd->job), "range");
+	g_return_if_fail (f != NULL);
+	r = gtk_object_get_data (GTK_OBJECT (r), "range");
+	if (r) gtk_container_remove (GTK_CONTAINER (f), r);
 
-	gtk_container_add (GTK_CONTAINER (gpd->range_container), custom);
-	gpd->range = custom;
-}
-
-
-static void
-replace (GtkWidget **o, GtkWidget *n)
-{
-	if (*o) gtk_object_unref ((GtkObject *) *o);
-
-	*o=n;
+	gtk_widget_show (custom);
+	gtk_container_add (GTK_CONTAINER (f), custom);
+	gtk_object_set_data (GTK_OBJECT (f), "range", custom);
 }
 
 /**
@@ -299,105 +352,226 @@ void
 gnome_print_dialog_construct_range_any (GnomePrintDialog *gpd, gint flags, GtkWidget *range_widget,
 					const guchar *currentlabel, const guchar *rangelabel)
 {
-	GtkWidget *table;
-	GtkWidget *current = NULL, *all = NULL, *range = NULL, *selection = NULL;
-	int row;
-	guint accel_key;
-	Private *p;
-	GSList *group = NULL;
+	GtkWidget *f, *r;
 
 	g_return_if_fail (gpd != NULL);
 	g_return_if_fail (GNOME_IS_PRINT_DIALOG (gpd));
-	g_return_if_fail (gpd->range_container != NULL);
-	g_return_if_fail (!(flags & GNOME_PRINT_RANGE_SELECTION && flags & GNOME_PRINT_RANGE_SELECTION_UNSENSITIVE));
-	p = gpd->priv;
+	g_return_if_fail (!range_widget || GTK_IS_WIDGET (range_widget));
+	g_return_if_fail (!(range_widget && !(flags & GNOME_PRINT_RANGE_RANGE)));
+	g_return_if_fail (!(!range_widget && (flags & GNOME_PRINT_RANGE_RANGE)));
+	g_return_if_fail (!((flags & GNOME_PRINT_RANGE_SELECTION) && (flags & GNOME_PRINT_RANGE_SELECTION_UNSENSITIVE)));
 
-	row=0;
-	if (flags&GNOME_PRINT_RANGE_SELECTION) row++;
-	if (flags&GNOME_PRINT_RANGE_SELECTION_UNSENSITIVE) row++;
-	if (flags&GNOME_PRINT_RANGE_ALL) row++;
-	if (flags&GNOME_PRINT_RANGE_RANGE) row++;
-	if (flags&GNOME_PRINT_RANGE_CURRENT) row++;
+	f = gtk_object_get_data (GTK_OBJECT (gpd->job), "range");
+	g_return_if_fail (f != NULL);
+	r = gtk_object_get_data (GTK_OBJECT (r), "range");
+	if (r) gtk_container_remove (GTK_CONTAINER (f), r);
 
-	table = gtk_table_new (row, 2, FALSE);
-	row=0;
-	if (flags & GNOME_PRINT_RANGE_CURRENT) {
-		current = gtk_radio_button_new_with_label (group, "");
-		accel_key = gtk_label_parse_uline
-			(GTK_LABEL (GTK_BIN (current)->child), currentlabel);
-		gtk_table_attach(GTK_TABLE(table), current, 0, 1, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
-		group = gtk_radio_button_group((GtkRadioButton *)current);
-		if (accel_key != GDK_VoidSymbol)
-			gtk_widget_add_accelerator (current, "clicked",
-						    gpd->range_accel_group,
-						    accel_key,
-						    GDK_MOD1_MASK, 0);
-		row++;
+	r = gpd_create_range (flags, range_widget, currentlabel, rangelabel);
+
+	if (r) {
+		gtk_widget_show (r);
+		gtk_container_add (GTK_CONTAINER (f), r);
 	}
 
-	if (flags&GNOME_PRINT_RANGE_ALL) {
-		all = gtk_radio_button_new_with_label(group, "");
-		accel_key = gtk_label_parse_uline
-			(GTK_LABEL (GTK_BIN (all)->child), _("_All"));
-		gtk_table_attach(GTK_TABLE(table), all, 0, 1, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
-		group = gtk_radio_button_group((GtkRadioButton *)all);
-		if (accel_key != GDK_VoidSymbol)
-			gtk_widget_add_accelerator (all, "clicked",
-						    gpd->range_accel_group,
-						    accel_key,
-						    GDK_MOD1_MASK, 0);
-		row++;
+	gtk_object_set_data (GTK_OBJECT (f), "range", r);
+}
+
+/**
+ * gnome_print_dialog_construct_range_page:
+ * @gpd: An initialise GnomePrintDialog, which can contain a range.
+ * @flags: Option flags.  See gnome_print_dialog_construct_any().
+ * @start: First page which may be printed.
+ * @end: Last page which may be printed.
+ * @currentlabel: Label text for current option.
+ * @rangelabel: Label text for range option.
+ * 
+ * Construct a generic page/sheet range area.
+ **/
+void
+gnome_print_dialog_construct_range_page (GnomePrintDialog *gpd, gint flags, gint start, gint end,
+					 const guchar *currentlabel, const guchar *rangelabel)
+{
+	GtkWidget *hbox;
+
+	hbox = NULL;
+
+	if (flags & GNOME_PRINT_RANGE_RANGE) {
+		GtkWidget *l, *sb;
+		GtkObject *a;
+
+		hbox = gtk_hbox_new (FALSE, 3);
+		gtk_widget_show (hbox);
+
+		l = gtk_label_new (_("From:"));
+		gtk_widget_show (l);
+		gtk_box_pack_start (GTK_BOX (hbox), l, FALSE, FALSE, 0);
+
+		a = gtk_adjustment_new (start, start, end, 1, 10, 10);
+		gtk_object_set_data (GTK_OBJECT (hbox), "from", a);
+		sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 1, 0.0);
+		gtk_widget_show (sb);
+		gtk_box_pack_start (GTK_BOX (hbox), sb, FALSE, FALSE, 0);
+
+		l = gtk_label_new (_("To:"));
+		gtk_widget_show (l);
+		gtk_box_pack_start (GTK_BOX (hbox), l, FALSE, FALSE, 0);
+
+		a = gtk_adjustment_new (start, start, end, 1, 10, 10);
+		gtk_object_set_data (GTK_OBJECT (hbox), "to", a);
+		sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 1, 0.0);
+		gtk_widget_show (sb);
+		gtk_box_pack_start (GTK_BOX (hbox), sb, FALSE, FALSE, 0);
 	}
 
-	if (flags&GNOME_PRINT_RANGE_RANGE) {
-		range = gtk_radio_button_new_with_label(group, "");
-		accel_key = gtk_label_parse_uline
-			(GTK_LABEL (GTK_BIN (range)->child), rangelabel);
-		gtk_table_attach(GTK_TABLE(table), range, 0, 1, row, row+1, GTK_FILL, GTK_SHRINK, 0, 0);
-		gtk_table_attach(GTK_TABLE(table), range_widget, 1, 2, row, row+1, GTK_FILL, 0, 0, 0);
-		group = gtk_radio_button_group((GtkRadioButton *)range);
-		if (accel_key != GDK_VoidSymbol)
-			gtk_widget_add_accelerator (range, "clicked",
-						    gpd->range_accel_group,
-						    accel_key,
-						    GDK_MOD1_MASK, 0);
-		row++;
+	gnome_print_dialog_construct_range_any (gpd, flags, hbox, currentlabel, rangelabel);
+}
+
+/**
+ * gnome_print_dialog_get_range:
+ * @gpd: A GnomePrintDialog with a range display.
+ * 
+ * Return the range option selected by the user.  This is a bitmask
+ * with only 1 bit set, out of:
+ *
+ * GNOME_PRINT_RANGE_CURRENT: The current option selected.
+ * GNOME_PRINT_RANGE_ALL: The all option selected.
+ * GNOME_PRINT_RANGE_RANGE The range option selected.
+ * GNOME_PRINT_RANGE_SELECTION: The selection option selected.
+ * 
+ * Return value: A bitmask with one option set.
+ **/
+GnomePrintRangeType
+gnome_print_dialog_get_range (GnomePrintDialog *gpd)
+{
+	GtkWidget *f, *r, *b;
+
+	g_return_val_if_fail (gpd != NULL, 0);
+	g_return_val_if_fail (GNOME_IS_PRINT_DIALOG (gpd), 0);
+
+	f = gtk_object_get_data (GTK_OBJECT (gpd->job), "range");
+	g_return_val_if_fail (f != NULL, 0);
+	r = gtk_object_get_data (GTK_OBJECT (f), "range");
+	g_return_val_if_fail (r != NULL, 0);
+
+	b = gtk_object_get_data (GTK_OBJECT (b), "current");
+	if (b && GTK_IS_TOGGLE_BUTTON (b) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) return GNOME_PRINT_RANGE_CURRENT;
+	b = gtk_object_get_data (GTK_OBJECT (b), "all");
+	if (b && GTK_IS_TOGGLE_BUTTON (b) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) return GNOME_PRINT_RANGE_ALL;
+	b = gtk_object_get_data (GTK_OBJECT (b), "range");
+	if (b && GTK_IS_TOGGLE_BUTTON (b) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) return GNOME_PRINT_RANGE_RANGE;
+	b = gtk_object_get_data (GTK_OBJECT (b), "selection");
+	if (b && GTK_IS_TOGGLE_BUTTON (b) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) return GNOME_PRINT_RANGE_SELECTION;
+
+	return 0;
+}
+
+/**
+ * gnome_print_dialog_get_range_page:
+ * @gpd: A GnomePrintDialog with a page range display.
+ * @start: Return for the user-specified start page.
+ * @end: Return for the user-specified end page.
+ * 
+ * Retrieves the user choice for range type and range, if the user
+ * has requested a range of pages to print.
+ * 
+ * Return value: A bitmask with the user-selection set.  See
+ * gnome_print_dialog_get_range().
+ **/
+gint
+gnome_print_dialog_get_range_page (GnomePrintDialog *gpd, gint *start, gint *end)
+{
+	gint mask;
+
+	g_return_val_if_fail (gpd != NULL, 0);
+	g_return_val_if_fail (GNOME_IS_PRINT_DIALOG(gpd), 0);
+
+	mask = gnome_print_dialog_get_range (gpd);
+
+	if (mask & GNOME_PRINT_RANGE_RANGE) {
+		GtkObject *f, *r, *a;
+		f = gtk_object_get_data (GTK_OBJECT (gpd->job), "range");
+		g_return_val_if_fail (f != NULL, 0);
+		r = gtk_object_get_data (GTK_OBJECT (f), "range");
+		g_return_val_if_fail (r != NULL, 0);
+		a = gtk_object_get_data (GTK_OBJECT (r), "from");
+		g_return_val_if_fail (a && GTK_IS_ADJUSTMENT (a), 0);
+		if (start) *start = (gint) gtk_adjustment_get_value (GTK_ADJUSTMENT (a));
+		a = gtk_object_get_data (GTK_OBJECT (r), "to");
+		g_return_val_if_fail (a && GTK_IS_ADJUSTMENT (a), 0);
+		if (end) *end = (gint) gtk_adjustment_get_value (GTK_ADJUSTMENT (a));
 	}
 
-	if (flags&GNOME_PRINT_RANGE_SELECTION) {
-		selection = gtk_radio_button_new_with_label(group, "");
-		accel_key = gtk_label_parse_uline
-			(GTK_LABEL (GTK_BIN (selection)->child),
-			 _("_Selection"));
-		gtk_table_attach(GTK_TABLE(table), selection, 0, 1, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
-		group = gtk_radio_button_group((GtkRadioButton *)selection);
-		if (accel_key != GDK_VoidSymbol)
-			gtk_widget_add_accelerator (selection, "clicked",
-						    gpd->range_accel_group,
-						    accel_key,
-						    GDK_MOD1_MASK, 0);
-		row++;
-	}
-	if (flags&GNOME_PRINT_RANGE_SELECTION_UNSENSITIVE) {
-		selection = gtk_radio_button_new_with_label(group, "");
-		accel_key = gtk_label_parse_uline
-			(GTK_LABEL (GTK_BIN (selection)->child),
-			 _("_Selection"));
-		gtk_widget_set_sensitive (GTK_WIDGET(selection), FALSE);
-		gtk_table_attach(GTK_TABLE (table), selection, 0, 1, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
-		group = gtk_radio_button_group ((GtkRadioButton *)selection);
-		row++;
+	return mask;
+}
+
+/**
+ * gnome_print_dialog_get_copies:
+ * @gpd: A GnomePrintDialog with a copies display.
+ * @copies: Return for the number of copies.
+ * @collate: Return for collation flag.
+ * 
+ * Retrieves the number of copies and collation indicator from
+ * the print dialogue.  If the print dialogue does not have a
+ * copies indicator, then a default of 1 copy is returned.
+ **/
+void
+gnome_print_dialog_get_copies (GnomePrintDialog *gpd, gint *copies, gint *collate)
+{
+	g_return_if_fail (gpd != NULL);
+	g_return_if_fail (GNOME_IS_PRINT_DIALOG (gpd));
+
+	if (copies) *copies = 1;
+	if (collate) *collate = FALSE;
+
+	if (gpd->job) {
+		GnomePrintCopiesSelection *c;
+
+		c = gtk_object_get_data (GTK_OBJECT (gpd->job), "copies");
+		if (c && GNOME_IS_PRINT_COPIES_SELECTION (c)) {
+			if (copies) *copies = gnome_print_copies_selection_get_copies (c);
+			if (collate) *collate = gnome_print_copies_selection_get_collate (c);
+		}
 	}
+}
 
-	replace(&p->current, current);
-	replace(&p->all, all);
-	replace(&p->range, range);
-	replace(&p->selection, selection);
+/**
+ * gnome_print_dialog_set_copies:
+ * @gpd: A GnomePrintDialog with a copies display.
+ * @copies: New number of copies.
+ * @collate: New collation status.
+ * 
+ * Sets the print copies and collation status in the print dialogue.
+ **/
+void
+gnome_print_dialog_set_copies (GnomePrintDialog *gpd, gint copies, gint collate)
+{
+	GnomePrintCopiesSelection *c;
 
-	gtk_widget_show_all(table);
+	g_return_if_fail (gpd != NULL);
+	g_return_if_fail (GNOME_IS_PRINT_DIALOG (gpd));
+	g_return_if_fail (gpd->job != NULL);
+	c = gtk_object_get_data (GTK_OBJECT (gpd->job), "copies");
+	g_return_if_fail (c && GNOME_IS_PRINT_COPIES_SELECTION (c));
 
-	gnome_print_dialog_construct_range_custom(gpd, table);
-	gpd->range_type = GNOME_PRINT_RANGETYPE_CUSTOM;
+	gnome_print_copies_selection_set_copies (c, copies, collate);
+}
+
+/**
+ * gnome_print_dialog_get_printer:
+ * @gpd: An initialised GnomePrintDialog.
+ * 
+ * Retrieve the user-requested printer from the printer area of
+ * the print dialogue.
+ * 
+ * Return value: The user-selected printer.
+ **/
+GnomePrintConfig *
+gnome_print_dialog_get_config (GnomePrintDialog *gpd)
+{
+	g_return_val_if_fail (gpd != NULL, NULL);
+	g_return_val_if_fail (GNOME_IS_PRINT_DIALOG (gpd), NULL);
+
+	return gnome_print_config_ref (gpd->config);
 }
 
 #if 0
@@ -468,156 +642,9 @@ gnome_print_dialog_construct_range_date (GnomePrintDialog *gpd, gint flags, time
 	replace(&p->range_end, range_end);
 
 	gnome_print_dialog_construct_range_any(gpd, flags, table, currentlabel, rangelabel);
-	gpd->range_type = GNOME_PRINT_RANGETYPE_PAGES;
 }
 #endif
 
-/**
- * gnome_print_dialog_construct_range_page:
- * @gpd: An initialise GnomePrintDialog, which can contain a range.
- * @flags: Option flags.  See gnome_print_dialog_construct_any().
- * @start: First page which may be printed.
- * @end: Last page which may be printed.
- * @currentlabel: Label text for current option.
- * @rangelabel: Label text for range option.
- * 
- * Construct a generic page/sheet range area.
- **/
-void
-gnome_print_dialog_construct_range_page (GnomePrintDialog *gpd, gint flags, gint start, gint end,
-					 const guchar *currentlabel, const guchar *rangelabel)
-{
-	GtkWidget *hbox = NULL, *label;
-	GtkWidget *range_start = NULL, *range_end = NULL;
-	GtkAdjustment *adj_start, *adj_end;
-	GtkEntry *entry;
-	guint label_key;
-	Private *p;
-
-	g_return_if_fail(gpd != NULL);
-	g_return_if_fail(GNOME_IS_PRINT_DIALOG(gpd));
-	g_return_if_fail(gpd->range_container != NULL);
-
-	p = gpd->priv;
-
-	if (flags&GNOME_PRINT_RANGE_RANGE) {
-		hbox = gtk_hbox_new(FALSE, 3);
-		label = gtk_label_new("");
-		label_key = gtk_label_parse_uline (GTK_LABEL (label),
-						   _("f_rom:"));
-		gtk_box_pack_start((GtkBox *)hbox, label, FALSE, FALSE, 0);
-
-		adj_start = (GtkAdjustment *)gtk_adjustment_new(start, start, end, 1, 10, 10);
-		range_start = gtk_spin_button_new (adj_start, 1, 0.0);
-		gtk_box_pack_start((GtkBox *)hbox, range_start, FALSE, FALSE, 0);
-		if (label_key != GDK_VoidSymbol) {
-			entry = &GTK_SPIN_BUTTON (range_start)->entry;
-			gtk_widget_add_accelerator (GTK_WIDGET (entry),
-						    "grab_focus",
-						    gpd->range_accel_group,
-						    label_key, GDK_MOD1_MASK,
-						    0);
-		}
-	
-		label = gtk_label_new("");
-		label_key = gtk_label_parse_uline (GTK_LABEL (label),
-						   _("_to:"));
-		gtk_box_pack_start((GtkBox *)hbox, label, FALSE, FALSE, 0);
-
-		adj_end = (GtkAdjustment *)gtk_adjustment_new(end, start, end, 1, 10, 10);
-		range_end = gtk_spin_button_new (adj_end, 1, 0.0);
-		gtk_box_pack_start((GtkBox *)hbox, range_end, FALSE, FALSE, 0);
-		if (label_key != GDK_VoidSymbol) {
-			entry = &GTK_SPIN_BUTTON (range_end)->entry;
-			gtk_widget_add_accelerator (GTK_WIDGET (entry),
-						    "grab_focus",
-						    gpd->range_accel_group,
-						    label_key, GDK_MOD1_MASK,
-						    0);
-		}
-
-	}
-
-	replace(&p->range_start, range_start);
-	replace(&p->range_end, range_end);
-
-	gnome_print_dialog_construct_range_any(gpd, flags, hbox, currentlabel, rangelabel);
-	gpd->range_type = GNOME_PRINT_RANGETYPE_PAGES;
-}
-
-/**
- * gnome_print_dialog_get_range:
- * @gpd: A GnomePrintDialog with a range display.
- * 
- * Return the range option selected by the user.  This is a bitmask
- * with only 1 bit set, out of:
- *
- * GNOME_PRINT_RANGE_CURRENT: The current option selected.
- * GNOME_PRINT_RANGE_ALL: The all option selected.
- * GNOME_PRINT_RANGE_RANGE The range option selected.
- * GNOME_PRINT_RANGE_SELECTION: The selection option selected.
- * 
- * Return value: A bitmask with one option set.
- **/
-GnomePrintRangeType
-gnome_print_dialog_get_range (GnomePrintDialog *gpd)
-{
-	Private *p;
-
-	g_return_val_if_fail(gpd != NULL, 0);
-	g_return_val_if_fail(GNOME_IS_PRINT_DIALOG(gpd), 0);
-	g_return_val_if_fail(gpd->range_container != NULL, 0);
-
-	p = gpd->priv;
-
-	if (p->current!=NULL)
-	     if (GTK_TOGGLE_BUTTON(p->current)->active)
-		  return GNOME_PRINT_RANGE_CURRENT;
-	if (p->all!=NULL)
-	     if (GTK_TOGGLE_BUTTON(p->all)->active)
-		  return GNOME_PRINT_RANGE_ALL;
-	if (p->range!=NULL)
-	     if (GTK_TOGGLE_BUTTON(p->range)->active)
-		  return GNOME_PRINT_RANGE_RANGE;
-	if (p->selection!=NULL)
-	     if (GTK_TOGGLE_BUTTON(p->selection)->active)
-		  return GNOME_PRINT_RANGE_SELECTION;
-	return 0;
-}
-
-/**
- * gnome_print_dialog_get_range_page:
- * @gpd: A GnomePrintDialog with a page range display.
- * @start: Return for the user-specified start page.
- * @end: Return for the user-specified end page.
- * 
- * Retrieves the user choice for range type and range, if the user
- * has requested a range of pages to print.
- * 
- * Return value: A bitmask with the user-selection set.  See
- * gnome_print_dialog_get_range().
- **/
-int
-gnome_print_dialog_get_range_page (GnomePrintDialog *gpd, int *start, int *end)
-{
-	int mask;
-	Private *p;
-
-	g_return_val_if_fail(gpd != NULL, 0);
-	g_return_val_if_fail(GNOME_IS_PRINT_DIALOG(gpd), 0);
-	g_return_val_if_fail(gpd->range_container != NULL, 0);
-	/*	g_return_val_if_fail(gpd->range_type != GNOME_PRINT_RANGETYPE_PAGES, 0);*/
-
-	p = gpd->priv;
-
-	mask = gnome_print_dialog_get_range(gpd);
-	if (mask & GNOME_PRINT_RANGE_RANGE) {
-		*start = gtk_spin_button_get_value_as_int ((GtkSpinButton *)p->range_start);
-		*end = gtk_spin_button_get_value_as_int ((GtkSpinButton *)p->range_end);
-	}
-	return mask;
-}
-
 #if 0
 /**
  * gnome_print_dialog_get_range_date:
@@ -641,7 +668,6 @@ gnome_print_dialog_get_range_date (GnomePrintDialog *gpd, time_t *start, time_t
 	g_return_val_if_fail(gpd != NULL, 0);
 	g_return_val_if_fail(GNOME_IS_PRINT_DIALOG(gpd), 0);
 	g_return_val_if_fail(gpd->range_container != NULL, 0);
-	g_return_val_if_fail(gpd->range_type != GNOME_PRINT_RANGETYPE_DATES, 0);
 
 	p = gpd->priv;
 
@@ -654,53 +680,5 @@ gnome_print_dialog_get_range_date (GnomePrintDialog *gpd, time_t *start, time_t
 }
 #endif
 
-/**
- * gnome_print_dialog_get_copies:
- * @gpd: A GnomePrintDialog with a copies display.
- * @copies: Return for the number of copies.
- * @collate: Return for collation flag.
- * 
- * Retrieves the number of copies and collation indicator from
- * the print dialogue.  If the print dialogue does not have a
- * copies indicator, then a default of 1 copy is returned.
- **/
-void
-gnome_print_dialog_get_copies (GnomePrintDialog *gpd, gint *copies, gint *collate)
-{
-	if (gpd->copies) {
-		if (copies) *copies = gnome_print_copies_selection_get_copies (GNOME_PRINT_COPIES_SELECTION (gpd->copies));
-		if (collate) *collate = gnome_print_copies_selection_get_collate (GNOME_PRINT_COPIES_SELECTION (gpd->copies));
-	} else {
-		if (copies) *copies = 1;
-		if (collate) *collate = FALSE;
-	}
-}
 
-/**
- * gnome_print_dialog_set_copies:
- * @gpd: A GnomePrintDialog with a copies display.
- * @copies: New number of copies.
- * @collate: New collation status.
- * 
- * Sets the print copies and collation status in the print dialogue.
- **/
-void
-gnome_print_dialog_set_copies (GnomePrintDialog *gpd, gint copies, gint collate)
-{
-	if (gpd->copies) gnome_print_copies_selection_set_copies (GNOME_PRINT_COPIES_SELECTION (gpd->copies), copies, collate);
-}
 
-/**
- * gnome_print_dialog_get_printer:
- * @gpd: An initialised GnomePrintDialog.
- * 
- * Retrieve the user-requested printer from the printer area of
- * the print dialogue.
- * 
- * Return value: The user-selected printer.
- **/
-GnomePrintConfig *
-gnome_print_dialog_get_config (GnomePrintDialog *gpd)
-{
-	return gnome_printer_selection_get_config (GNOME_PRINTER_SELECTION (gpd->printer));
-}
