@@ -153,12 +153,16 @@ render_page (GnomePrintMasterPreview *pmp, gint page)
 	priv->preview = gnome_print_preview_new_full (config, priv->canvas, transform, &region);
 	gnome_print_config_unref (config);
 
+#if 0
 	meta = gnome_print_master_get_context (priv->master);
 	data = gnome_print_meta_get_buffer (GNOME_PRINT_META (meta));
 	len = gnome_print_meta_get_length (GNOME_PRINT_META (meta));
 	g_object_unref (G_OBJECT (meta));
 
 	gnome_print_meta_render_data_page (priv->preview, data, len, page, TRUE);
+#else
+	gnome_print_master_render_page (priv->master, priv->preview, page, TRUE);
+#endif
 
 	return GNOME_PRINT_OK;
 }
@@ -167,13 +171,12 @@ static gint
 goto_page (GnomePrintMasterPreview *pmp, gint page)
 {
 	GPMPPrivate *priv;
-	gchar *text;
+	guchar c[32];
 
 	priv = (GPMPPrivate *) pmp->priv;
 
-	text = g_strdup_printf ("%d", page + 1);
-	gtk_entry_set_text (GTK_ENTRY (priv->page_entry), text);
-	g_free (text);
+	g_snprintf (c, 32, "%d", page + 1);
+	gtk_entry_set_text (GTK_ENTRY (priv->page_entry), c);
 	
 	if (page != priv->current_page) {
 		priv->current_page = page;
@@ -722,6 +725,7 @@ create_toplevel (GnomePrintMasterPreview *pmp)
 	GPMPPrivate *priv;
 	GnomePrintConfig *config;
 	gint width, height;
+	const GnomePrintUnit *unit;
 	gdouble val;
 
 	priv = (GPMPPrivate *) pmp->priv;
@@ -729,10 +733,16 @@ create_toplevel (GnomePrintMasterPreview *pmp)
 	config = gnome_print_master_get_config (priv->master);
 
 	val = 21.0 * 72.0 / 2.54;
-	gnome_print_config_get_double (config, GNOME_PRINT_KEY_PAPER_WIDTH, &val);
+	if (gnome_print_config_get_length (config, GNOME_PRINT_KEY_PAPER_WIDTH, &val, &unit)) {
+		g_print ("len %g %s\n", val, unit->abbr);
+		gnome_print_convert_distance (&val, unit, GNOME_PRINT_PS_UNIT);
+		g_print ("abs %g\n", val);
+	}
 	width = val;
 	val = 29.7 * 72.0 / 2.54;
-	gnome_print_config_get_double (config, GNOME_PRINT_KEY_PAPER_HEIGHT, &val);
+	if (gnome_print_config_get_length (config, GNOME_PRINT_KEY_PAPER_HEIGHT, &val, &unit)) {
+		gnome_print_convert_distance (&val, unit, GNOME_PRINT_PS_UNIT);
+	}
 	height = val;
 
 	if (priv->landscape) {
