@@ -90,8 +90,6 @@ gpa_settings_selector_init (GPASettingsSelector *selector)
 
 	selector->printer = NULL;
 	selector->settings = NULL;
-	selector->pmid = 0;
-	selector->smid = 0;
 
 	selector->settingslist = NULL;
 }
@@ -109,17 +107,13 @@ gpa_settings_selector_destroy (GtkObject *object)
 	}
 
 	if (ss->printer) {
-		if (ss->pmid) gtk_signal_disconnect (GTK_OBJECT (ss->printer), ss->pmid);
 		gpa_node_unref (ss->printer);
 		ss->printer = NULL;
-		ss->pmid = 0;
 	}
 
 	if (ss->settings) {
-		if (ss->smid) gtk_signal_disconnect (GTK_OBJECT (ss->settings), ss->smid);
 		gpa_node_unref (ss->settings);
 		ss->settings = NULL;
-		ss->smid = 0;
 	}
 
 	if (((GtkObjectClass *) parent_class)->destroy)
@@ -133,23 +127,11 @@ gpa_settings_selector_construct (GPAWidget *widget)
 	GPANode *node;
 
 	ss = GPA_SETTINGS_SELECTOR (widget);
-	node = widget->node;
+	node = GNOME_PRINT_CONFIG_NODE (widget->config);
 
 	ss->printer = gpa_node_get_path_node (node, "Printer");
 	ss->settings = gpa_node_get_path_node (node, "Settings");
 
-	gtk_signal_connect_while_alive (GTK_OBJECT (node), "modified",
-					GTK_SIGNAL_FUNC (gpa_ss_root_modified), ss, GTK_OBJECT (ss));
-
-	if (ss->printer) {
-		ss->pmid = gtk_signal_connect (GTK_OBJECT (ss->printer), "modified",
-					       GTK_SIGNAL_FUNC (gpa_ss_printer_modified), ss);
-	}
-	if (ss->settings) {
-		ss->smid = gtk_signal_connect (GTK_OBJECT (ss->settings), "modified",
-					       GTK_SIGNAL_FUNC (gpa_ss_settings_modified), ss);
-	}
-
 	gpa_ss_rebuild_menu (ss);
 
 	return TRUE;
@@ -167,15 +149,11 @@ gpa_ss_root_modified (GPANode *config, GPASettingsSelector *ss)
 	printer = gpa_node_get_path_node (config, "Printer");
 	if (printer != ss->printer) {
 		if (ss->printer) {
-			if (ss->pmid) gtk_signal_disconnect (GTK_OBJECT (ss->printer), ss->pmid);
 			gpa_node_unref (ss->printer);
 			ss->printer = NULL;
-			ss->pmid = 0;
 		}
 		if (printer) {
 			ss->printer = printer;
-			ss->pmid = gtk_signal_connect (GTK_OBJECT (ss->printer), "modified",
-						       GTK_SIGNAL_FUNC (gpa_ss_printer_modified), ss);
 		}
 		/* fixme: rebuild menu? maybe only if !printer */
 	} else {
