@@ -32,7 +32,7 @@ static gint gpa_transport_selector_construct (GPAWidget *widget);
 static void gpa_ts_printer_modified (GPANode *printer, GPATransportSelector *ts);
 static void gpa_ts_rebuild_widget (GPATransportSelector *ts);
 
-static void gpa_ts_entry_changed (GtkEntry *e, GPATransportSelector *ts);
+static void gpa_ts_entry_changed (GtkEntry *entry, GPAWidget *gpw);
 static void gpa_ts_menuitem_activate (GtkWidget *widget, gint index);
 
 static void gpa_ts_select_transport (GPATransportSelector *ts, const gchar *id);
@@ -74,6 +74,16 @@ gpa_transport_selector_class_init (GPATransportSelectorClass *klass)
 	gpa_class->construct = gpa_transport_selector_construct;
 }
 
+static void
+gpa_ts_entry_changed (GtkEntry *entry, GPAWidget *gpw)
+{
+	const guchar *text;
+
+	text = gtk_entry_get_text (entry);
+
+	gnome_print_config_set (gpw->config, "Settings.Transport.Backend.FileName", text);
+}
+
 static void
 gpa_transport_selector_init (GPATransportSelector *ts)
 {
@@ -87,9 +97,8 @@ gpa_transport_selector_init (GPATransportSelector *ts)
 
 	/* Create filename entry */
 	ts->entry = gtk_entry_new ();
+	gtk_signal_connect (GTK_OBJECT (ts->entry), "changed", GTK_SIGNAL_FUNC (gpa_ts_entry_changed), ts);
 	gtk_box_pack_start (GTK_BOX (ts->hbox), ts->entry, TRUE, TRUE, 0);
-	gtk_signal_connect (GTK_OBJECT (ts->entry), "changed",
-			    GTK_SIGNAL_FUNC (gpa_ts_entry_changed), ts);
 
 	/* Create label */
 	ts->label = gtk_label_new (deflabel);
@@ -128,11 +137,7 @@ gpa_transport_selector_construct (GPAWidget *gpaw)
 
 	ts = GPA_TRANSPORT_SELECTOR (gpaw);
 
-	ts->printer = gpa_node_get_path_node (gpaw->node, "Printer");
-	if (ts->printer) {
-		gtk_signal_connect_while_alive (GTK_OBJECT (ts->printer), "modified",
-						GTK_SIGNAL_FUNC (gpa_ts_printer_modified), ts, GTK_OBJECT (ts));
-	}
+	ts->printer = gpa_node_get_path_node (GNOME_PRINT_CONFIG_NODE (gpaw->config), "Printer");
 
 	gpa_ts_rebuild_widget (ts);
 }
@@ -153,7 +158,7 @@ gpa_ts_rebuild_widget (GPATransportSelector *ts)
 	guchar *defid;
 	GSList *l;
 
-	node = GPA_WIDGET (ts)->node;
+	node = GNOME_PRINT_CONFIG_NODE (GPA_WIDGET (ts)->config);
 
 	/* Cleanup old state */
 	while (ts->transportlist) {
@@ -259,18 +264,6 @@ gpa_ts_menuitem_activate (GtkWidget *widget, gint index)
 	g_free (value);
 }
 
-static void
-gpa_ts_entry_changed (GtkEntry *e, GPATransportSelector *ts)
-{
-	GPANode *node;
-	const gchar *text;
-
-	node = ((GPAWidget *) ts)->node;
-
-	text = gtk_entry_get_text (e);
-	gpa_node_set_path_value (node, "Settings.Transport.Backend.FileName", text);
-}
-
 static void
 gpa_ts_select_transport (GPATransportSelector *ts, const gchar *id)
 {
@@ -278,12 +271,12 @@ gpa_ts_select_transport (GPATransportSelector *ts, const gchar *id)
 
 	gpaw = GPA_WIDGET (ts);
 
-	gpa_node_set_path_value (gpaw->node, "Settings.Transport.Backend", id);
+	gnome_print_config_set (gpaw->config, "Settings.Transport.Backend", id);
 
 	if (!strcmp (id, "file")) {
 		guchar *filename;
 		/* Current transport is file */
-		filename = gpa_node_get_path_value (gpaw->node, "Settings.Transport.Backend.FileName");
+		filename = gnome_print_config_get (gpaw->config, "Settings.Transport.Backend.FileName");
 		if (filename) {
 			gtk_entry_set_text (GTK_ENTRY (ts->entry), filename);
 			g_free (filename);
@@ -296,7 +289,7 @@ gpa_ts_select_transport (GPATransportSelector *ts, const gchar *id)
 	} else if (!strcmp (id, "lpr")) {
 		guchar *lp;
 		/* Current transport is lpr */
-		lp = gpa_node_get_path_value (gpaw->node, "Settings.Transport.Backend.Printer");
+		lp = gnome_print_config_get (gpaw->config, "Settings.Transport.Backend.Printer");
 		if (lp && *lp) {
 			gchar *text;
 			text = g_strdup_printf (_("Line printer: %s"), lp);
